
%
% function [JAC] = brussel(N, L)
%
% N : size
% L : parameter. (L  = 1  or  0.5)
%
function [JAC] = brussel(N, L)

N = sqrt(N / 2.);
A = 2.;
B = 5.45;
Dx = 0.004;
Dy = 0.008;

% steady state : U = [A, B/A, A, B/A, ..]
for k=1:N*N,
  U(2*k-1) = A;
  U(2*k) = B/A;
end
%
%
JAC = sparse( 2*N*N, 2*N*N );
%
for i = 1 : N
  for j = 1 : N
    ind = 2*i*N + 2*j - 2*N - 1;
    Sx = Dx * (N+1) * (N+1) / (L * L);
    Sy = Dy * (N+1) * (N+1) / (L * L);
    if ( i > 1 )
      JAC(ind, ind - 2*N) = Sx;
      JAC(ind+1, ind+1 - 2*N) = Sy;
    end
    if ( i < N )
      JAC(ind, ind + 2*N) = Sx;
      JAC(ind+1, ind+1 + 2*N) = Sy;
    end
    if ( j > 1 )
      JAC(ind, ind - 2) = Sx;
      JAC(ind+1, ind - 1) = Sy;
    end
    if ( j < N )
      JAC(ind, ind + 2) = Sx;
      JAC(ind+1, ind + 3) = Sy;
    end
    JAC(ind, ind) = -4. * Sx - (B+1) + 2. * U(ind) * U(ind+1);
    JAC(ind+1, ind+1) = -4. * Sy - U(ind) * U(ind);
    JAC(ind, ind+1) = U(ind) * U(ind);
    JAC(ind+1, ind) = - 2. * U(ind) * U(ind+1) + B;
  end
end
