 function [perm,sizes] = map4 (A, ndom, centers) 
%%%% function [perm,sizes] = map4 (A, ndom, centers) 
%% -----------------------------------------------------------------------
%%   finds domains given ndom centers -- by doing a level set expansion 
%% ---------------- vertex partitioner -- --------------------------------
%%      on entry:
%%      ---------
%%      n       = dimension of matrix 
%%      A       = sparse matrix or adjacency list  -- 
%%      ndom    = number of subdomains wanted
%%      centers = array of size ndom containing the labels of the
%%                "centers" of the ndom domains from which to do
%%                the expansions. 
%%      
%%      on return 
%%      --------- 
%%      perm  = list of nodes in each subdomain in sequence
%%      sizes = number of nodes in each of the subdomains found
%% 
%%-----------------------------------------------------------------------
     n = size(A,1);
     marker = zeros(n,1); 
     for i=1:ndom 
         list(i,1) = centers(i); 
         sizes(i) = 1; 
         marker(centers(i)) = i; 
         levst(i) = 0;
     end 
%%
     count = ndom ; 
     nstuck = 0 ; 
     newpt  = 1; 
%% ---------------------------------------------------------------------
%% 
  while (count < n) 
        if (nstuck < 0)          %% idom already determined from previous loop
           nstuck = 0 ; 
        else 
            idom = mindom(n,ndom,sizes) ;
        end
%%-----------------------------------------------------------------------
%%     begin level-set loop 
%%-----------------------------------------------------------------------
      prevcount = count; 
      last = sizes(idom) ;
      first = levst(idom); 
      levst(idom) = last; 
      lastlast=last;
%%  list traversal loop
      for nod=first+1:lastlast
      jnod = list(idom,nod) ; 
%%     adjacency list traversal loop 
          [i,j,r] = find(A(jnod,:));
          [i,j1,r] = find(A(:,jnod));
          j = union(i',j) ;
          for k=1:length(j)
              jcol = j(k) ; 
              if (marker(jcol) == 0) 
                last = last+1;
	        list(idom,last) = jcol; 
	        count = count+1;
                marker(jcol) = idom; 
              end
          end
      end
      sizes(idom) = last; 
%%-----------------------------------------------------------------------
%%     end level-set traversal --  
%%-----------------------------------------------------------------------
       if (prevcount == count) 
%%         sizes(idom) <0 indicates that set is stuck -- This locks it out
           sizes(idom) = -sizes(idom) ; 
           nstuck = nstuck+1; 
      end
%% take care of situations of deadlock        
       if (nstuck >= ndom) 
%%      reset sizes -- (unlock all sets) 
         for j=1:ndom
               if (sizes(j) < 0) 
                  sizes(j) = -sizes(j);
              end
         end
         nstuck = 0; 
%% 
         while (marker(newpt) >0) 
               newpt = newpt+1;
               if (newpt > n) 
                      error(' -- graph seems incorrect ');
               end
         end
%%----------------------------------------------------------------------- 
        idom = 0 ;
        isiz = n+1; 
        [i,j,r] = find(A(newpt,:));
        for k=1:length(j)
            jcol = j(k) ; 
            jdom = marker(jcol); 
            if (jdom > 0) 
                  nsize = sizes(jdom);
                  if (nsize < isiz) 
                     isiz = nsize;
                     idom = jdom; 
                  end
            end
        end 
%%-----------------------------------------------------------------------
        if (idom == 0) 
            idom =  mindom(n, ndom, sizes) ; 
            nstuck = -1;
        end 
        last = sizes(idom)+1; 
        list(idom,last) = newpt; 
        count = count+1;
        marker(newpt) = idom; 
        sizes(idom)=last;
     end 
  end 
%% wrap up
    bw = n+1; 
    sizes = abs(sizes);
    idx = 0; 
    for idom=1:ndom
        for k=1:sizes(idom)
            jnod = list(idom,k) ;
            [i,j,r] = find(A(jnod,:));
            [i,j1,r] = find(A(:,jnod));
            j = union(i',j) ;
            interf = 0; 
            for k=1:length(j)
               jcol = j(k) ; 
               if (marker(jcol) ~= idom)  %%% interf, point
	          interf = 1;
                  break; 
                end
             end
	     if (interf == 0) 
         	   idx = idx+1; 
                   perm(idx) = jnod;
             else
                   bw = bw-1;
                   perm(bw) = jnod; 
                   sizes(idom) = sizes(idom) - 1;
            end
        end
     end 
%% done 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     function mindom = mindom(n, ndom, sizes) 
%% -----------------------------------------------------------------------
%%      returns  the domain with smallest size
%% ----------------------------------------------------------------------- 
     isiz = n+1 ;
     mindom = 1; 
     for i=1:ndom
         nsize = sizes(i) ; 
         if (nsize > 0) 
            if (nsize < isiz)
            isiz = nsize; 
            mindom = i; 
            end
         end
      end 
